@@ -23,6 +23,8 @@
 #include "gasyncresult.h"
 #include "gcancellable.h"
 
+#include "glibintl.h"
+
 /**
  * SECTION:gtask
  * @short_description: Cancellable synchronous or asynchronous task
@@ -540,13 +542,14 @@ struct _GTask {
   GDestroyNotify task_data_destroy;
 
   GMainContext *context;
-  guint64 creation_time;
+  gint64 creation_time;
   gint priority;
   GCancellable *cancellable;
   gboolean check_cancellable;
 
   GAsyncReadyCallback callback;
   gpointer callback_data;
+  gboolean completed;
 
   GTaskThreadFunc task_func;
   GMutex lock;
@@ -574,6 +577,11 @@ struct _GTaskClass
   GObjectClass parent_class;
 };
 
+typedef enum
+{
+  PROP_COMPLETED = 1,
+} GTaskProperty;
+
 static void g_task_thread_pool_resort (void);
 
 static void g_task_async_result_iface_init (GAsyncResultIface *iface);
@@ -1074,9 +1082,17 @@ static void
 g_task_return_now (GTask *task)
 {
   g_main_context_push_thread_default (task->context);
-  task->callback (task->source_object,
-                  G_ASYNC_RESULT (task),
-                  task->callback_data);
+
+  if (task->callback != NULL)
+    {
+      task->callback (task->source_object,
+                      G_ASYNC_RESULT (task),
+                      task->callback_data);
+    }
+
+  task->completed = TRUE;
+  g_object_notify (G_OBJECT (task), "completed");
+
   g_main_context_pop_thread_default (task->context);
 }
 
@@ -1103,7 +1119,7 @@ g_task_return (GTask           *task,
   if (type == G_TASK_RETURN_SUCCESS)
     task->result_set = TRUE;
 
-  if (task->synchronous || !task->callback)
+  if (task->synchronous)
     return;
 
   /* Normally we want to invoke the task's callback when its return
@@ -1274,16 +1290,22 @@ g_task_start_task_thread (GTask           *task,
           return;
         }
 
+      /* This introduces a reference count loop between the GTask and
+       * GCancellable, but is necessary to avoid a race on finalising the GTask
+       * between task_thread_cancelled() (in one thread) and
+       * g_task_thread_complete() (in another).
+       *
+       * Accordingly, the signal handler *must* be removed once the task has
+       * completed.
+       */
       g_signal_connect_data (task->cancellable, "cancelled",
                              G_CALLBACK (task_thread_cancelled),
                              g_object_ref (task),
                              task_thread_cancelled_disconnect_notify, 0);
     }
 
-  g_thread_pool_push (task_pool, g_object_ref (task), &task->error);
-  if (task->error)
-    task->thread_complete = TRUE;
-  else if (g_private_get (&task_private))
+  g_thread_pool_push (task_pool, g_object_ref (task), NULL);
+  if (g_private_get (&task_private))
     {
       /* This thread is being spawned from another GTask thread, so
        * bump up max-threads so we don't starve.
@@ -1348,6 +1370,7 @@ g_task_run_in_thread (GTask           *task,
  * Normally this is used with tasks created with a %NULL
  * `callback`, but note that even if the task does
  * have a callback, it will not be invoked when @task_func returns.
+ * #GTask:completed will be set to %TRUE just before this function returns.
  *
  * Since: 2.36
  */
@@ -1366,6 +1389,11 @@ g_task_run_in_thread_sync (GTask           *task,
     g_cond_wait (&task->cond, &task->lock);
 
   g_mutex_unlock (&task->lock);
+
+  /* Notify of completion in this thread. */
+  task->completed = TRUE;
+  g_object_notify (G_OBJECT (task), "completed");
+
   g_object_unref (task);
 }
 
@@ -1712,6 +1740,26 @@ g_task_had_error (GTask *task)
   return FALSE;
 }
 
+/**
+ * g_task_get_completed:
+ * @task: a #GTask.
+ *
+ * Gets the value of #GTask:completed. This changes from %FALSE to %TRUE after
+ * the task’s callback is invoked, and will return %FALSE if called from inside
+ * the callback.
+ *
+ * Returns: %TRUE if the task has completed, %FALSE otherwise.
+ *
+ * Since: 2.44
+ */
+gboolean
+g_task_get_completed (GTask *task)
+{
+  g_return_val_if_fail (G_IS_TASK (task), FALSE);
+
+  return task->completed;
+}
+
 /**
  * g_task_is_valid:
  * @result: (type Gio.AsyncResult): A #GAsyncResult
@@ -1784,12 +1832,50 @@ g_task_thread_pool_resort (void)
   g_thread_pool_set_sort_function (task_pool, g_task_compare_priority, NULL);
 }
 
+static void
+g_task_get_property (GObject    *object,
+                     guint       prop_id,
+                     GValue     *value,
+                     GParamSpec *pspec)
+{
+  GTask *task = G_TASK (object);
+
+  switch ((GTaskProperty) prop_id)
+    {
+    case PROP_COMPLETED:
+      g_value_set_boolean (value, task->completed);
+      break;
+    }
+}
+
 static void
 g_task_class_init (GTaskClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->get_property = g_task_get_property;
   gobject_class->finalize = g_task_finalize;
+
+  /**
+   * GTask:completed:
+   *
+   * Whether the task has completed, meaning its callback (if set) has been
+   * invoked. This can only happen after g_task_return_pointer(),
+   * g_task_return_error() or one of the other return functions have been called
+   * on the task.
+   *
+   * This property is guaranteed to change from %FALSE to %TRUE exactly once.
+   *
+   * The #GObject::notify signal for this change is emitted in the same main
+   * context as the task’s callback, immediately after that callback is invoked.
+   *
+   * Since: 2.44
+   */
+  g_object_class_install_property (gobject_class, PROP_COMPLETED,
+    g_param_spec_boolean ("completed",
+                          P_("Task completed"),
+                          P_("Whether the task has completed yet"),
+                          FALSE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static gpointer
