@@ -13,9 +13,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "gtesttlsbackend.h"
@@ -81,10 +79,8 @@ enum
   PROP_CERTIFICATE_0,
 
   PROP_CERT_CERTIFICATE,
-  PROP_CERT_CERTIFICATE_BYTES,
   PROP_CERT_CERTIFICATE_PEM,
   PROP_CERT_PRIVATE_KEY,
-  PROP_CERT_PRIVATE_KEY_BYTES,
   PROP_CERT_PRIVATE_KEY_PEM,
   PROP_CERT_ISSUER
 };
@@ -96,6 +92,15 @@ G_DEFINE_TYPE_WITH_CODE (GTestTlsCertificate, g_test_tls_certificate, G_TYPE_TLS
 			 G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
 						g_test_tls_certificate_initable_iface_init);)
 
+static GTlsCertificateFlags
+g_test_tls_certificate_verify (GTlsCertificate     *cert,
+                               GSocketConnectable  *identity,
+                               GTlsCertificate     *trusted_ca)
+{
+  /* For now, all of the tests expect the certificate to verify */
+  return 0;
+}
+
 static void
 g_test_tls_certificate_get_property (GObject    *object,
 				      guint       prop_id,
@@ -135,9 +140,7 @@ g_test_tls_certificate_set_property (GObject      *object,
       cert->key_pem = g_value_dup_string (value);
       break;
     case PROP_CERT_CERTIFICATE:
-    case PROP_CERT_CERTIFICATE_BYTES:
     case PROP_CERT_PRIVATE_KEY:
-    case PROP_CERT_PRIVATE_KEY_BYTES:
     case PROP_CERT_ISSUER:
       /* ignore */
       break;
@@ -157,19 +160,20 @@ g_test_tls_certificate_finalize (GObject *object)
 }
 
 static void
-g_test_tls_certificate_class_init (GTestTlsCertificateClass *certificate_class)
+g_test_tls_certificate_class_init (GTestTlsCertificateClass *test_class)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (certificate_class);
+  GObjectClass *gobject_class = G_OBJECT_CLASS (test_class);
+  GTlsCertificateClass *certificate_class = G_TLS_CERTIFICATE_CLASS (test_class);
 
   gobject_class->get_property = g_test_tls_certificate_get_property;
   gobject_class->set_property = g_test_tls_certificate_set_property;
   gobject_class->finalize = g_test_tls_certificate_finalize;
 
+  certificate_class->verify = g_test_tls_certificate_verify;
+
   g_object_class_override_property (gobject_class, PROP_CERT_CERTIFICATE, "certificate");
-  g_object_class_override_property (gobject_class, PROP_CERT_CERTIFICATE_BYTES, "certificate-bytes");
   g_object_class_override_property (gobject_class, PROP_CERT_CERTIFICATE_PEM, "certificate-pem");
   g_object_class_override_property (gobject_class, PROP_CERT_PRIVATE_KEY, "private-key");
-  g_object_class_override_property (gobject_class, PROP_CERT_PRIVATE_KEY_BYTES, "private-key-bytes");
   g_object_class_override_property (gobject_class, PROP_CERT_PRIVATE_KEY_PEM, "private-key-pem");
   g_object_class_override_property (gobject_class, PROP_CERT_ISSUER, "issuer");
 }
